import { storageLocalGet, storageLocalSet } from "./storage.js";
import { extractAllowRules } from "./utils/utils.js";

/**
 * Upgrade data from previous installations.
 *
 * @param guardian {Guardian} a guardian to update status
 * @returns {Promise<unknown>} a promise that resolves when the upgrade is complete
 */
export function upgrade(guardian) {
  return new Promise((resolve) => {
    storageLocalGet("ub_mv3").then((upgradeRecord) => {
      if (upgradeRecord["ub_mv3"]) {
        // no upgrade needed
        guardian.setStatus("upgrade.timestamp", upgradeRecord["ub_mv3"]);
        guardian.setStatus("upgrade.status", "skipped");
        resolve([]);
        return;
      }

      const started = Date.now();
      const mv3 = new Date().toISOString();
      storageLocalSet({ ub_mv3: mv3 }).then(() =>
        guardian.setStatus("upgrade.stamp", mv3)
      );
      guardian.setStatus("upgrade.record", JSON.stringify(upgradeRecord));
      guardian.setStatus("upgrade.status", "started");

      const connection = indexedDB.open("EWE", 1);

      connection.onsuccess = (e) => {
        const allowList = new Set();
        guardian.setStatus("upgrade.connection", true);
        const database = e.target.result;
        try {
          const names = database.objectStoreNames;
          if (names.length == 0) {
            guardian.setStatus("upgrade.status", "empty");
            return;
          }
          const evaluated = [];
          const scanned = [];
          for (let i = 0; i < names.length; i++) {
            const name = names.item(i);
            evaluated.push(name);
            guardian.setStatus("upgrade.stores", JSON.stringify(evaluated));
            if (name === "file-contents") {
              scanned.push(name);
              guardian.setStatus("upgrade.contents", true);
              const transaction = database.transaction("file-contents");
              const objectStore = transaction.objectStore("file-contents");
              const values = objectStore.getAll();
              values.onsuccess = (e) => {
                let reviewed = 0;
                guardian.setStatus("upgrade.status", "importing");
                guardian.setStatus("upgrade.import", true);
                e.target.result.forEach((result) => {
                  const lines = new TextDecoder("utf-8").decode(result);
                  extractAllowRules(allowList, lines.split(/\r\n|\r|\n/));
                  reviewed++;
                });
                indexedDB.deleteDatabase("EWE");
                guardian.setStatus("upgrade.db.deleted", true);
                guardian.setStatus("upgrade.status", "upgraded");
                guardian.setStatus("upgrade.elapsed", Date.now() - started);
                guardian.setStatus("upgrade.allow.size", allowList.size);
                guardian.setStatus("upgrade.reviewed", reviewed);
                resolve(Array.from(allowList));
              };
              values.onerror = (e) => {
                // Still need to clean up the database
                indexedDB.deleteDatabase("EWE");
                guardian.setStatus("upgrade.db.deleted", true);
                guardian.setStatus("upgrade.values.error", true);
                guardian.setStatus("upgrade.status", "failed");
                guardian.setStatus("upgrade.error", e.target.errorCode);
                guardian.setLastError(e.target.errorCode);
                resolve([]);
              };
            }
          }
          if (scanned.length === 0) {
            indexedDB.deleteDatabase("EWE");
            guardian.setStatus("upgrade.db.deleted", true);
            guardian.setStatus("upgrade.status", "no-stores");
            guardian.setStatus("upgrade.elapsed", Date.now() - started);
            guardian.setStatus("upgrade.evaluated", JSON.stringify(evaluated));
            resolve([]);
          }
        } catch (err) {
          // the database likely doesn't actually exist - do we need to do this
          indexedDB.deleteDatabase("EWE");
          guardian.setStatus("upgrade.db.deleted", true);
          guardian.setStatus("upgrade.status", "failed");
          guardian.setStatus("upgrade.error", err);
          guardian.setLastError(err);
          resolve([]);
        }
      };
      connection.onerror = (e) => {
        indexedDB.deleteDatabase("EWE");
        guardian.setStatus("upgrade.status", "failed");
        guardian.setStatus("upgrade.connection", false);
        guardian.setStatus("upgrade.error", e.target.errorCode);
        guardian.setLastError(e.target.errorCode);
        resolve([]);
      };
    });
  });
}
